@@ -2,6 +2,10 @@
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <string.h>
 #include <locale.h>
 #include <limits.h>
@@ -16,6 +20,7 @@ Eina_List *img_dirs = NULL;
 Eina_List *fnt_dirs = NULL;
 Eina_List *defines = NULL;
 char      *file_in = NULL;
+char      *tmp_dir = NULL;
 char      *file_out = NULL;
 char      *progname = NULL;
 int        verbose = 0;
@@ -37,6 +42,7 @@ main_help(void)
       "\n"
       "-id image/directory      Add a directory to look in for relative path images\n"
       "-fd font/directory       Add a directory to look in for relative path fonts\n"
+      "-td temp/directory       Directory to store temporary files\n"
       "-v                       Verbose output\n"
       "-no-lossy                Do NOT allow images to be lossy\n"
       "-no-comp                 Do NOT allow images to be stored with lossless compression\n"
@@ -58,6 +64,9 @@ main(int argc, char **argv)
 
    eina_init();
 
+   tmp_dir = getenv("TMPDIR");
+
+
    progname = argv[0];
    for (i = 1; i < argc; i++)
      {
@@ -92,6 +101,12 @@ main(int argc, char **argv)
 	     i++;
 	     fnt_dirs = eina_list_append(fnt_dirs, argv[i]);
 	  }
+	else if ((!strcmp(argv[i], "-td") || !strcmp(argv[i], "--tmp_dir")) && (i < (argc - 1)))
+	  {
+	     i++;
+             if (!tmp_dir)
+               tmp_dir = argv[i];
+	  }
 	else if ((!strcmp(argv[i], "-min-quality")) && (i < (argc - 1)))
 	  {
 	     i++;
