@@ -1,14 +1,30 @@
 /*
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
+/* EDJE - EFL graphical design and layout library based on Evas
+ * Copyright (C) 2008 Cedric Bail
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library;
+ * if not, see <http://www.gnu.org/licenses/>.
+ */
+
+
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#include <string.h>
-#include <sys/stat.h>
-
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
 #elif defined __GNUC__
@@ -26,8 +42,16 @@ extern "C"
 void *alloca (size_t);
 #endif
 
+#include <string.h>
+#include <sys/stat.h>
+
 #include "edje_private.h"
 
+
+/**
+ * @cond
+ */
+
 static Eina_Hash   *_edje_file_hash = NULL;
 static int          _edje_file_cache_size = 16;
 static Eina_List   *_edje_file_cache = NULL;
@@ -67,6 +91,29 @@ _edje_file_coll_open(Edje_File *edf, const char *coll)
 	free(data);
      }
 
+   snprintf(buf, sizeof(buf), "lua_scripts/%i", id);
+   data = eet_read(edf->ef, buf, &size);
+
+   if (data)
+     {
+	int err_code;
+
+	//printf("lua chunk size: %d\n", size);
+	edc->L = _edje_lua_new_thread(_edje_lua_state_get()); // gets freed in 'edje_load::_edje_collection_free'
+	_edje_lua_new_reg(edc->L, -1, edc); // gets freed in 'edje_load::_edje_collectoin_free'
+
+	if ((err_code = luaL_loadbuffer(edc->L, data, size, "edje_lua_script")))
+	  {
+	     if (err_code == LUA_ERRSYNTAX)
+	       printf("lua load syntax error: %s\n", lua_tostring(edc->L, -1));
+	     else if (err_code == LUA_ERRMEM)
+	       printf("lua load memory allocation error: %s\n", lua_tostring(edc->L, -1));
+	  }
+	if (lua_pcall(edc->L, 0, 0, 0))
+	  printf("lua call error: %s\n", lua_tostring(edc->L, -1));
+	free(data);
+     }
+   
    edc->part = eina_stringshare_add(coll);
    edc->references = 1;
    if (!edf->collection_hash)
@@ -415,8 +462,41 @@ _edje_file_cache_shutdown(void)
 
 
 
+/**
+ * @endcond
+ */
 
+/*============================================================================*
+ *                                 Global                                     *
+ *============================================================================*/
 
+/*============================================================================*
+ *                                   API                                      *
+ *============================================================================*/
+
+/**
+ * @addtogroup Edje_cache_Group Cache
+ *
+ * @brief These functions provide an abstraction layer between the
+ * application code and the interface, while allowing extremely
+ * flexible dynamic layouts and animations.
+ *
+ * @{
+ */
+
+/**
+ * @brief Set the file cache size.
+ *
+ * @param count The file cache size in edje file units. Default is 16.
+ *
+ * This function sets the file cache size. Edje keeps this cache in
+ * order to prevent duplicates of edje file entries in memory. The
+ * file cache size can be retrieved with edje_file_cache_get().
+ *
+ * @see edje_file_cache_get()
+ * @see edje_file_cache_flush()
+ *
+ */
 
 EAPI void
 edje_file_cache_set(int count)
@@ -426,12 +506,38 @@ edje_file_cache_set(int count)
    _edje_cache_file_clean();
 }
 
+/**
+ * @brief Return the file cache size.
+ *
+ * @return The file cache size in edje file units. Default is 16.
+ *
+ * This function returns the file cache size set by
+ * edje_file_cache_set().
+ *
+ * @see edje_file_cache_set()
+ * @see edje_file_cache_flush()
+ *
+ */
+
 EAPI int
 edje_file_cache_get(void)
 {
    return _edje_file_cache_size;
 }
 
+/**
+ * @brief Clean the file cache.
+ *
+ * @return The file cache size.
+ *
+ * This function cleans the file cache entries, but keeps this cache's
+ * size to the last value set.
+ *
+ * @see edje_file_cache_set()
+ * @see edje_file_cache_get()
+ *
+ */
+
 EAPI void
 edje_file_cache_flush(void)
 {
@@ -443,6 +549,21 @@ edje_file_cache_flush(void)
    _edje_file_cache_size = ps;
 }
 
+/**
+ * @brief Set the collection cache size.
+ *
+ * @param count The collection cache size, in edje object units. Default is 16.
+ *
+ * This function sets the collection cache size. Edje keeps this cache
+ * in order to prevent duplicates of edje {collection,group,part}
+ * entries in memory. The collection cache size can be retrieved with
+ * edje_collection_cache_get().
+ *
+ * @see edje_collection_cache_get()
+ * @see edje_collection_cache_flush()
+ *
+ */
+
 EAPI void
 edje_collection_cache_set(int count)
 {
@@ -456,12 +577,38 @@ edje_collection_cache_set(int count)
    /* FIXME: freach in file hash too! */
 }
 
+/**
+ * @brief Return the collection cache size.
+ *
+ * @return The collection cache size, in edje object units. Default is 16.
+ *
+ * This function returns the collection cache size set by
+ * edje_collection_cache_set().
+ *
+ * @see edje_collection_cache_set()
+ * @see edje_collection_cache_flush()
+ *
+ */
+
 EAPI int
 edje_collection_cache_get(void)
 {
    return _edje_collection_cache_size;
 }
 
+/**
+ * @brief Clean the collection cache.
+ *
+ * @return The collection cache size.
+ *
+ * This function cleans the collection cache, but keeps this cache's
+ * size to the last value set.
+ *
+ * @see edje_collection_cache_set()
+ * @see edje_collection_cache_get()
+ *
+ */
+
 EAPI void
 edje_collection_cache_flush(void)
 {
@@ -476,3 +623,8 @@ edje_collection_cache_flush(void)
    /* FIXME: freach in file hash too! */
    _edje_collection_cache_size = ps;
 }
+
+/**
+ *
+ * @}
+ */
