@@ -296,6 +296,7 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
    old_swallows = _edje_swallows_collect(ed);
 
    if (_edje_script_only(ed)) _edje_script_only_shutdown(ed);
+   if (_edje_lua_script_only(ed)) _edje_lua_script_only_shutdown(ed);
    _edje_file_del(ed);
 
    if (ed->path) eina_stringshare_del(ed->path);
@@ -304,7 +305,18 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
    ed->group = eina_stringshare_add(group);
 
    ed->load_error = EDJE_LOAD_ERROR_NONE;
-  _edje_file_add(ed);
+   _edje_file_add(ed);
+
+   if (ed->file && ed->file->external_dir)
+     {
+       const Edje_External_Directory_Entry *ext;
+       const Eina_List *n;
+
+       EINA_LIST_FOREACH(ed->file->external_dir->entries, n, ext)
+         {
+            edje_module_load(ext->entry);
+         }
+     }
 
    _edje_textblock_styles_add(ed);
    _edje_textblock_style_all_update(ed);
@@ -316,6 +328,11 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 	     ed->load_error = EDJE_LOAD_ERROR_NONE;
 	     _edje_script_only_init(ed);
 	  }
+	else if (ed->collection->lua_script_only)
+	  {
+	     ed->load_error = EDJE_LOAD_ERROR_NONE;
+	     _edje_lua_script_only_init(ed);
+	  }
 	else
 	  {
 	     Eina_List *l;
@@ -346,12 +363,29 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 		  Edje_Real_Part *rp;
 
 		  ep = eina_list_data_get(l);
-		  rp = calloc(1, sizeof(Edje_Real_Part));
+		  rp = eina_mempool_malloc(_edje_real_part_mp, sizeof(Edje_Real_Part));
 		  if (!rp)
 		    {
 		       ed->load_error = EDJE_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
 		       return 0;
 		    }
+
+		  memset(rp, 0, sizeof (Edje_Real_Part));
+
+		  if ((ep->dragable.x != 0) || (ep->dragable.y != 0))
+		    {
+		       rp->drag = calloc(1, sizeof (Edje_Real_Part_Drag));
+		       if (!rp->drag)
+			 {
+			    ed->load_error = EDJE_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
+			    free(rp);
+			    return 0;
+			 }
+
+		       rp->drag->step.x = ep->dragable.step_x;
+		       rp->drag->step.y = ep->dragable.step_y;
+		    }
+
 		  rp->edje = ed;
 		  _edje_ref(rp->edje);
 		  rp->part = ep;
@@ -360,35 +394,45 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 		  rp->chosen_description = rp->param1.description;
 		  if (!rp->param1.description)
 		    printf("EDJE ERROR: no default part description!\n");
-		  if (ep->type == EDJE_PART_TYPE_RECTANGLE)
-		    rp->object = evas_object_rectangle_add(ed->evas);
-		  else if (ep->type == EDJE_PART_TYPE_IMAGE)
-		    rp->object = evas_object_image_add(ed->evas);
-		  else if (ep->type == EDJE_PART_TYPE_TEXT)
+
+		  switch (ep->type)
 		    {
-		       _edje_text_part_on_add(ed, rp);
-		       rp->object = evas_object_text_add(ed->evas);
-		       evas_object_text_font_source_set(rp->object, ed->path);
+		     case EDJE_PART_TYPE_RECTANGLE:
+			rp->object = evas_object_rectangle_add(ed->evas);
+			break;
+		     case EDJE_PART_TYPE_IMAGE:
+			rp->object = evas_object_image_add(ed->evas);
+			break;
+		     case EDJE_PART_TYPE_TEXT:
+			_edje_text_part_on_add(ed, rp);
+			rp->object = evas_object_text_add(ed->evas);
+			evas_object_text_font_source_set(rp->object, ed->path);
+			break;
+		     case EDJE_PART_TYPE_SWALLOW:
+		     case EDJE_PART_TYPE_GROUP:
+		     case EDJE_PART_TYPE_EXTERNAL:
+			rp->object = evas_object_rectangle_add(ed->evas);
+			evas_object_color_set(rp->object, 0, 0, 0, 0);
+			evas_object_pass_events_set(rp->object, 1);
+			evas_object_pointer_mode_set(rp->object, EVAS_OBJECT_POINTER_MODE_NOGRAB);
+			break;
+		     case EDJE_PART_TYPE_TEXTBLOCK:
+			rp->object = evas_object_textblock_add(ed->evas);
+			break;
+		     case EDJE_PART_TYPE_GRADIENT:
+			rp->object = evas_object_gradient_add(ed->evas);
+			break;
+		     case EDJE_PART_TYPE_BOX:
+			rp->object = evas_object_box_add(ed->evas);
+			break;
+		     case EDJE_PART_TYPE_TABLE:
+			rp->object = evas_object_table_add(ed->evas);
+			break;
+		     default:
+			printf("EDJE ERROR: wrong part type %i!\n", ep->type);
+			break;
 		    }
-		  else if (ep->type == EDJE_PART_TYPE_SWALLOW || ep->type == EDJE_PART_TYPE_GROUP)
-		    {
-		       rp->object = evas_object_rectangle_add(ed->evas);
-		       evas_object_color_set(rp->object, 0, 0, 0, 0);
-		       evas_object_pass_events_set(rp->object, 1);
-		       evas_object_pointer_mode_set(rp->object, EVAS_OBJECT_POINTER_MODE_NOGRAB);
-		    }
-		  else if (ep->type == EDJE_PART_TYPE_TEXTBLOCK)
-		    rp->object = evas_object_textblock_add(ed->evas);
-		  else if (ep->type == EDJE_PART_TYPE_GRADIENT)
-		    rp->object = evas_object_gradient_add(ed->evas);
-		  else if (ep->type == EDJE_PART_TYPE_BOX)
-		    {
-		       rp->object = evas_object_box_add(ed->evas);
-		    }
-		  else if (ep->type == EDJE_PART_TYPE_TABLE)
-		    rp->object = evas_object_table_add(ed->evas);
-		  else
-		    printf("EDJE ERROR: wrong part type %i!\n", ep->type);
+
 		  if (rp->object)
 		    {
 		       evas_object_smart_member_add(rp->object, ed->obj);
@@ -415,8 +459,6 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 		       if (rp->part->clip_to_id < 0)
 			 evas_object_clip_set(rp->object, ed->clipper);
 		    }
-		  rp->drag.step.x = ep->dragable.step_x;
-		  rp->drag.step.y = ep->dragable.step_y;
 		  rp->gradient_id = -1;
 	       }
 	     if (n > 0)
@@ -443,7 +485,6 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 			 rp->param1.rel2_to_x = ed->table_parts[rp->param1.description->rel2.id_x % ed->table_parts_size];
 		       if (rp->param1.description->rel2.id_y >= 0)
 			 rp->param1.rel2_to_y = ed->table_parts[rp->param1.description->rel2.id_y % ed->table_parts_size];
-		       _edje_text_part_on_add_clippers(ed, rp);
 		       if (rp->part->clip_to_id >= 0)
 			 {
 			    rp->clip_to = ed->table_parts[rp->part->clip_to_id % ed->table_parts_size];
@@ -454,9 +495,12 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 				 evas_object_clip_set(rp->object, rp->clip_to->object);
 			      }
 			 }
-		       if (rp->part->dragable.confine_id >= 0)
-			 rp->confine_to = ed->table_parts[rp->part->dragable.confine_id % ed->table_parts_size];
-		       
+		       if (rp->drag)
+			 {
+			    if (rp->part->dragable.confine_id >= 0)
+			      rp->drag->confine_to = ed->table_parts[rp->part->dragable.confine_id % ed->table_parts_size];
+			 }
+
 		       /* replay events for dragable */
 		       if (rp->part->dragable.events_id >= 0)
 			 {
@@ -467,7 +511,7 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 				!rp->events_to->part->dragable.y)
 			      rp->events_to = NULL;
 			 }
-		       
+
 		       rp->swallow_params.min.w = 0;
 		       rp->swallow_params.min.w = 0;
 		       rp->swallow_params.max.w = -1;
@@ -519,11 +563,17 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 		  rp = ed->table_parts[i];
 		  evas_object_show(rp->object);
 		  if (_edje_block_break(ed)) break;
-		  if (rp->part->dragable.x < 0) rp->drag.val.x = 1.0;
-		  if (rp->part->dragable.y < 0) rp->drag.val.x = 1.0;
-		  _edje_dragable_pos_set(ed, rp, rp->drag.val.x, rp->drag.val.y);
+		  if (rp->drag)
+		    {
+		       if (rp->part->dragable.x < 0) rp->drag->val.x = 1.0;
+		       if (rp->part->dragable.y < 0) rp->drag->val.x = 1.0;
+		       _edje_dragable_pos_set(ed, rp, rp->drag->val.x, rp->drag->val.y);
+		    }
 	       }
 	     ed->dirty = 1;
+#ifdef EDJE_CALC_CACHE
+	     ed->all_part_change = 1;
+#endif
 	     if ((evas_object_clipees_get(ed->clipper)) &&
 		 (evas_object_visible_get(obj)))
 	       evas_object_show(ed->clipper);
@@ -542,21 +592,36 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 		  const char *source = NULL;
 		  
 		  rp = ed->table_parts[i];
-		  if (rp->part->type != EDJE_PART_TYPE_GROUP &&
-		      rp->part->type != EDJE_PART_TYPE_BOX &&
-		      rp->part->type != EDJE_PART_TYPE_TABLE) continue;
-		  if (rp->part->type == EDJE_PART_TYPE_GROUP)
-		    source = rp->part->source;
-		  else if (rp->part->type == EDJE_PART_TYPE_BOX ||
-			   rp->part->type == EDJE_PART_TYPE_TABLE)
+
+		  switch (rp->part->type)
 		    {
-		       if (rp->part->items)
-			 {
-			    curr_item = rp->part->items;
-			    pack_it = curr_item->data;
-			    source = pack_it->source;
-			 }
+		     case EDJE_PART_TYPE_GROUP:
+			source = rp->part->source;
+			break;
+		     case EDJE_PART_TYPE_BOX:
+		     case EDJE_PART_TYPE_TABLE:
+			if (rp->part->items)
+			  {
+			     curr_item = rp->part->items;
+			     pack_it = curr_item->data;
+			     source = pack_it->source;
+			  }
+			break;
+		     case EDJE_PART_TYPE_EXTERNAL:
+			  {
+			     Evas_Object *child_obj;
+			     child_obj = _edje_external_type_add(rp->part->source, evas_object_evas_get(ed->obj), ed->obj, rp->part->default_desc->external_params);
+			     if (child_obj)
+			       {
+				  _edje_real_part_swallow(rp, child_obj);
+				  rp->param1.external_params = _edje_external_params_parse(child_obj, rp->param1.description->external_params);
+			       }
+			  }
+			continue;
+		     default:
+			continue;
 		    }
+
 		  while (source)
 		    {
 		       Eina_List *l;
@@ -652,11 +717,10 @@ _edje_object_file_set_internal(Evas_Object *obj, const char *file, const char *g
 	     
 	     if (group_path_started)
 	       {
-		  while (group_path)
-		    {
-		       eina_stringshare_del(eina_list_data_get(group_path));
-		       group_path = eina_list_remove_list(group_path, group_path);
-		    }
+		  const char *str;
+
+		  EINA_LIST_FREE(group_path, str)
+		    eina_stringshare_del(str);
 	       }
 	     
 	     /* reswallow any swallows that existed before setting the file */
@@ -782,48 +846,57 @@ _edje_file_del(Edje *ed)
 	       _edje_entry_real_part_shutdown(rp);
 	     if (rp->object)
 	       {
-		  _edje_text_real_part_on_del(ed, rp);
 		  _edje_callbacks_del(rp->object);
 		  evas_object_del(rp->object);
 	       }
 	     if (rp->swallowed_object)
 	       {
-		  evas_object_smart_member_del(rp->swallowed_object);
-		  evas_object_event_callback_del(rp->swallowed_object,
-						 EVAS_CALLBACK_FREE,
-						 _edje_object_part_swallow_free_cb);
-		  evas_object_clip_unset(rp->swallowed_object);
-		  evas_object_data_del(rp->swallowed_object, "\377 edje.swallowing_part");
-		  if (rp->part->mouse_events)
-		     _edje_callbacks_del(rp->swallowed_object);
-
+                  _edje_real_part_swallow_clear(rp);
                   /* Objects swallowed by the app do not get deleted,
                    but those internally swallowed (GROUP type) do. */
-		  if (rp->part->type == EDJE_PART_TYPE_GROUP)
-		    evas_object_del(rp->swallowed_object);
-
+		  switch (rp->part->type)
+		    {
+		     case EDJE_PART_TYPE_EXTERNAL:
+			_edje_external_parsed_params_free(rp->swallowed_object, rp->param1.external_params);
+			if (rp->param2)
+			  _edje_external_parsed_params_free(rp->swallowed_object, rp->param2->external_params);
+		     case EDJE_PART_TYPE_GROUP:
+			evas_object_del(rp->swallowed_object);
+		     default:
+			break;
+		    }
 		  rp->swallowed_object = NULL;
 	       }
 	     if (rp->items)
 	       {
-		  /* all internal, for now */
-		  while (rp->items)
-		    {
-		       /* evas_box/table handles deletion of objects */
-		       /*evas_object_del(rp->items->data);*/
-		       rp->items = eina_list_remove_list(rp->items, rp->items);
-		    }
+		  /* evas_box/table handles deletion of objects */
+		  rp->items = eina_list_free(rp->items);
 	       }
 	     if (rp->text.text) eina_stringshare_del(rp->text.text);
 	     if (rp->text.font) eina_stringshare_del(rp->text.font);
 	     if (rp->text.cache.in_str) eina_stringshare_del(rp->text.cache.in_str);
 	     if (rp->text.cache.out_str) eina_stringshare_del(rp->text.cache.out_str);
 
-	     if (rp->custom.description)
-	       _edje_collection_free_part_description_free(rp->custom.description, ed->file->free_strings);
+	     if (rp->custom)
+               {
+                  if (ed->L)
+                    {
+                       _edje_lua_get_reg(ed->L, rp->custom->description);
+                       _edje_lua_free_reg(ed->L, lua_touserdata(ed->L, -1)); // created in edje_lua.c::_edje_lua_part_fn_custom_state
+                       lua_pop(ed->L, 1);
+                       _edje_lua_free_reg(ed->L, rp->custom->description); // created in edje_lua.c::_edje_lua_part_fn_custom_state
+                    }
+                  _edje_collection_free_part_description_free(rp->custom->description, ed->file->free_strings);
+               }
+
+	     /* Cleanup optional part. */
+	     free(rp->drag);
+
+	     eina_mempool_free(_edje_real_part_state_mp, rp->param2);
+	     eina_mempool_free(_edje_real_part_state_mp, rp->custom);
 
 	     _edje_unref(rp->edje);
-	     free(rp);
+	     eina_mempool_free(_edje_real_part_mp, rp);
 	  }
      }
    if ((ed->file) && (ed->collection))
@@ -834,9 +907,10 @@ _edje_file_del(Edje *ed)
 	_edje_textblock_styles_del(ed);
 	EINA_LIST_FOREACH(ed->collection->parts, l, ep)
 	  {
-	     _edje_text_part_on_del(ed, ep);
-	     _edje_color_class_on_del(ed, ep);
+	    _edje_text_part_on_del(ed, ep);
+	    _edje_color_class_on_del(ed, ep);
 	  }
+	
 	_edje_cache_coll_unref(ed->file, ed->collection);
 	ed->collection = NULL;
      }
@@ -847,29 +921,32 @@ _edje_file_del(Edje *ed)
      }
    if (ed->actions)
      {
-	while (ed->actions)
-	  {
-	     Edje_Running_Program *runp;
+	Edje_Running_Program *runp;
 
+	EINA_LIST_FREE(ed->actions, runp)
+	  {
 	     _edje_anim_count--;
-	     runp = eina_list_data_get(ed->actions);
-	     ed->actions = eina_list_remove(ed->actions, runp);
 	     free(runp);
 	  }
      }
    _edje_animators = eina_list_remove(_edje_animators, ed);
    if (ed->pending_actions)
      {
-	while (ed->pending_actions)
-	  {
-	     Edje_Pending_Program *pp;
+	Edje_Pending_Program *pp;
 
-	     pp = eina_list_data_get(ed->pending_actions);
-	     ed->pending_actions = eina_list_remove(ed->pending_actions, pp);
+	EINA_LIST_FREE(ed->pending_actions, pp)
+	  {
 	     ecore_timer_del(pp->timer);
 	     free(pp);
 	  }
      }
+   if (ed->L)
+     {
+	_edje_lua_free_reg(ed->L, ed); // created in edje_lua.c::_edje_lua_script_fn_new/_edje_lua_group_fn_new
+	_edje_lua_free_reg(ed->L, ed->L); // created in edje_program.c::_edje_program_run/edje_lua_script_only.c::_edje_lua_script_only_init
+	_edje_lua_free_thread(ed->L); // created in edje_program.c::_edje_program_run/edje_lua_script_only.c::_edje_lua_script_only_init
+	ed->L = NULL;
+     }
    if (ed->table_parts) free(ed->table_parts);
    ed->table_parts = NULL;
    ed->table_parts_size = 0;
@@ -881,7 +958,7 @@ _edje_file_del(Edje *ed)
  * Used to free the cached data values that are stored in the data_cache
  * hash table.
  */
-static Eina_Bool data_cache_free(const Eina_Hash *hash, const void *key, void *data, void *fdata)
+static Eina_Bool data_cache_free(const Eina_Hash *hash __UNUSED__, const void *key __UNUSED__, void *data, void *fdata)
 {
    Edje_File    *edf;
 
@@ -893,15 +970,15 @@ static Eina_Bool data_cache_free(const Eina_Hash *hash, const void *key, void *d
 void
 _edje_file_free(Edje_File *edf)
 {
+   Edje_Color_Class *ecc;
+   Edje_Data *edt;
+
    if (edf->font_dir)
      {
-	while (edf->font_dir->entries)
-	  {
-	     Edje_Font_Directory_Entry *fe;
+	Edje_Font_Directory_Entry *fe;
 
-	     fe = eina_list_data_get(edf->font_dir->entries);
-	     edf->font_dir->entries =
-	       eina_list_remove_list(edf->font_dir->entries, edf->font_dir->entries);
+	EINA_LIST_FREE(edf->font_dir->entries, fe)
+	  {
 	     eina_hash_del(edf->font_hash, fe->entry, edf);
 	     if (edf->free_strings && fe->path) eina_stringshare_del(fe->path);
 	     free(fe);
@@ -915,13 +992,10 @@ _edje_file_free(Edje_File *edf)
      }
    if (edf->image_dir)
      {
-	while (edf->image_dir->entries)
-	  {
-	     Edje_Image_Directory_Entry *ie;
+	Edje_Image_Directory_Entry *ie;
 
-	     ie = eina_list_data_get(edf->image_dir->entries);
-	     edf->image_dir->entries =
-	       eina_list_remove_list(edf->image_dir->entries, edf->image_dir->entries);
+	EINA_LIST_FREE(edf->image_dir->entries, ie)
+	  {
 	     if (edf->free_strings && ie->entry) eina_stringshare_del(ie->entry);
 	     free(ie);
 	  }
@@ -929,13 +1003,10 @@ _edje_file_free(Edje_File *edf)
      }
    if (edf->collection_dir)
      {
-	while (edf->collection_dir->entries)
-	  {
-	     Edje_Part_Collection_Directory_Entry *ce;
+	Edje_Part_Collection_Directory_Entry *ce;
 
-	     ce = eina_list_data_get(edf->collection_dir->entries);
-	     edf->collection_dir->entries =
-	       eina_list_remove_list(edf->collection_dir->entries, edf->collection_dir->entries);
+	EINA_LIST_FREE(edf->collection_dir->entries, ce)
+	  {
 	     if (edf->free_strings && ce->entry) eina_stringshare_del(ce->entry);
 	     free(ce);
 	  }
@@ -943,19 +1014,15 @@ _edje_file_free(Edje_File *edf)
      }
    if (edf->spectrum_dir)
      {
-	while (edf->spectrum_dir->entries)
+	Edje_Spectrum_Directory_Entry *se;
+
+	EINA_LIST_FREE(edf->spectrum_dir->entries, se)
 	  {
-	     Edje_Spectrum_Directory_Entry *se;
+	     Edje_Spectrum_Color *sc;
+
+	     EINA_LIST_FREE(se->color_list, sc)
+	       free(sc);
 
-	     se = eina_list_data_get(edf->spectrum_dir->entries);
-	     edf->spectrum_dir->entries =
-	       eina_list_remove_list(edf->spectrum_dir->entries, edf->spectrum_dir->entries);
-	     while (se->color_list)
-	       {
-		 free(eina_list_data_get(se->color_list));
-		  se->color_list =
-                    eina_list_remove_list(se->color_list, se->color_list);
-	       }
              if (edf->free_strings)
                {
                   if (se->entry) eina_stringshare_del(se->entry);
@@ -965,12 +1032,9 @@ _edje_file_free(Edje_File *edf)
 	  }
 	free(edf->spectrum_dir);
      }
-   while (edf->data)
-     {
-	Edje_Data *edt;
 
-	edt = eina_list_data_get(edf->data);
-	edf->data = eina_list_remove(edf->data, edt);
+   EINA_LIST_FREE(edf->data, edt)
+     {
         if (edf->free_strings)
           {
              if (edt->key) eina_stringshare_del(edt->key);
@@ -985,13 +1049,8 @@ _edje_file_free(Edje_File *edf)
 	edf->data_cache = NULL;
      }
 
-   while (edf->color_classes)
+   EINA_LIST_FREE(edf->color_classes, ecc)
      {
-	Edje_Color_Class *ecc;
-
-	ecc = eina_list_data_get(edf->color_classes);
-	edf->color_classes =
-          eina_list_remove_list(edf->color_classes, edf->color_classes);
 	if (edf->free_strings && ecc->name) eina_stringshare_del(ecc->name);
 	free(ecc);
      }
@@ -1024,12 +1083,14 @@ _edje_file_free(Edje_File *edf)
 void
 _edje_collection_free(Edje_File *edf, Edje_Part_Collection *ec)
 {
-   while (ec->programs)
+   Edje_Program *pr;
+   Edje_Part *ep;
+
+   EINA_LIST_FREE(ec->programs, pr)
      {
-	Edje_Program *pr;
+	Edje_Program_Target *prt;
+	Edje_Program_After *pa;
 
-	pr = eina_list_data_get(ec->programs);
-	ec->programs = eina_list_remove_list(ec->programs, ec->programs);
         if (edf->free_strings)
           {
              if (pr->name) eina_stringshare_del(pr->name);
@@ -1038,54 +1099,32 @@ _edje_collection_free(Edje_File *edf, Edje_Part_Collection *ec)
              if (pr->state) eina_stringshare_del(pr->state);
              if (pr->state2) eina_stringshare_del(pr->state2);
           }
-	while (pr->targets)
-	  {
-	     Edje_Program_Target *prt;
-
-	     prt = eina_list_data_get(pr->targets);
-	     pr->targets = eina_list_remove_list(pr->targets, pr->targets);
-	     free(prt);
-	  }
-	while (pr->after)
-	  {
-	     Edje_Program_After *pa;
-
-	     pa = eina_list_data_get(pr->after);
-	     pr->after = eina_list_remove_list(pr->after, pr->after);
-	     free(pa);
-	  }
+	EINA_LIST_FREE(pr->targets, prt)
+	  free(prt);
+	EINA_LIST_FREE(pr->after, pa)
+	  free(pa);
 	free(pr);
      }
-   while (ec->parts)
+   EINA_LIST_FREE(ec->parts, ep)
      {
-	Edje_Part *ep;
+	Edje_Part_Description *desc;
 
-	ep = eina_list_data_get(ec->parts);
-	ec->parts = eina_list_remove(ec->parts, ep);
 	if (edf->free_strings && ep->name) eina_stringshare_del(ep->name);
 	if (ep->default_desc)
 	  {
 	     _edje_collection_free_part_description_free(ep->default_desc, edf->free_strings);
 	     ep->default_desc = NULL;
 	  }
-	while (ep->other_desc)
-	  {
-	     Edje_Part_Description *desc;
-
-	     desc = eina_list_data_get(ep->other_desc);
-	     ep->other_desc = eina_list_remove(ep->other_desc, desc);
-	     _edje_collection_free_part_description_free(desc, edf->free_strings);
-	  }
+	EINA_LIST_FREE(ep->other_desc, desc)
+	  _edje_collection_free_part_description_free(desc, edf->free_strings);
 	free(ep);
      }
    if (ec->data)
      {
-	while (ec->data)
-	  {
-	     Edje_Data *edt;
+	Edje_Data *edt;
 
-	     edt = eina_list_data_get(ec->data);
-	     ec->data = eina_list_remove(ec->data, edt);
+	EINA_LIST_FREE(ec->data, edt)
+	  {
              if (edf->free_strings)
                {
                   if (edt->key) eina_stringshare_del(edt->key);
@@ -1106,20 +1145,24 @@ _edje_collection_free(Edje_File *edf, Edje_Part_Collection *ec)
      }
 #endif
    if (ec->script) embryo_program_free(ec->script);
+   if (ec->L)
+     {
+	_edje_lua_free_reg(ec->L, ec); // created in edje_cache.c::_edje_file_coll_open
+	_edje_lua_free_thread(ec->L); // created in edje_cache.c::_edje_file_coll_open
+	ec->L = NULL;
+     }
    free(ec);
 }
 
 void
 _edje_collection_free_part_description_free(Edje_Part_Description *desc, unsigned int free_strings)
 {
-   while (desc->image.tween_list)
-     {
-	Edje_Part_Image_Id *pi;
+   Edje_Part_Image_Id *pi;
 
-	pi = eina_list_data_get(desc->image.tween_list);
-	desc->image.tween_list = eina_list_remove(desc->image.tween_list, pi);
-	free(pi);
-     }
+   EINA_LIST_FREE(desc->image.tween_list, pi)
+     free(pi);
+   if (desc->external_params)
+     _edje_external_params_free(desc->external_params, free_strings);
    if (free_strings)
      {
 	if (desc->color_class)     eina_stringshare_del(desc->color_class);
@@ -1134,7 +1177,7 @@ _edje_collection_free_part_description_free(Edje_Part_Description *desc, unsigne
 }
 
 static Eina_Bool
-_edje_file_collection_hash_foreach(const Eina_Hash *hash, const void *key, void *data, void *fdata)
+_edje_file_collection_hash_foreach(const Eina_Hash *hash __UNUSED__, const void *key __UNUSED__, void *data, void *fdata)
 {
    Edje_File *edf;
    Edje_Part_Collection *coll;
@@ -1201,8 +1244,8 @@ _cb_signal_repeat(void *data, Evas_Object *obj, const char *signal, const char *
    Evas_Object	*parent;
    Edje		*ed;
    char		 new_src[4096]; /* XXX is this max reasonable? */
-   int		 length_parent = 0;
-   int		 length_source;
+   size_t	 length_parent = 0;
+   size_t	 length_source;
 
    parent = data;
    ed = _edje_fetch(obj);
